@@ -40,6 +40,7 @@ extern PyTypeObject TreeType;
 extern PyTypeObject CommitType;
 extern PyTypeObject BlobType;
 extern PyTypeObject TagType;
+extern PyObject *FileModeEnum;
 
 PyTypeObject ObjectType;
 
@@ -90,27 +91,6 @@ Object_id__get__(Object *self)
     return git_oid_to_python(Object__id(self));
 }
 
-PyDoc_STRVAR(Object_oid__doc__,
-    "The object id, an instance of the Oid type.\n"
-    "This attribute is deprecated, please use 'id'\n");
-
-PyObject *
-Object_oid__get__(Object *self)
-{
-    return Object_id__get__(self);
-}
-
-PyDoc_STRVAR(Object_hex__doc__,
-    "Hexadecimal representation of the object id. This is a shortcut for\n"
-    "Object.id.hex\n"
-    "This attribute is deprecated, please use 'id'\n");
-
-PyObject *
-Object_hex__get__(Object *self)
-{
-    return git_oid_to_py_str(Object__id(self));
-}
-
 
 PyDoc_STRVAR(Object_short_id__doc__,
     "An unambiguous short (abbreviated) hex Oid string for the object.");
@@ -146,7 +126,7 @@ PyDoc_STRVAR(Object_type_str__doc__,
 PyObject *
 Object_type_str__get__(Object *self)
 {
-    return to_path(git_object_type2string(Object__type(self)));
+    return PyUnicode_DecodeFSDefault(git_object_type2string(Object__type(self)));
 }
 
 PyDoc_STRVAR(Object__pointer__doc__, "Get the object's pointer. For internal use only.");
@@ -159,14 +139,14 @@ Object__pointer__get__(Object *self)
 }
 
 PyDoc_STRVAR(Object_name__doc__,
-             "Name (will be None if the object was not reached trough a tree)");
+             "Name (or None if the object was not reached through a tree)");
 PyObject *
 Object_name__get__(Object *self)
 {
     if (self->entry == NULL)
         Py_RETURN_NONE;
 
-    return to_path(git_tree_entry_name(self->entry));
+    return PyUnicode_DecodeFSDefault(git_tree_entry_name(self->entry));
 }
 
 PyDoc_STRVAR(Object_raw_name__doc__, "Name (bytes).");
@@ -181,14 +161,14 @@ Object_raw_name__get__(Object *self)
 }
 
 PyDoc_STRVAR(Object_filemode__doc__,
-             "Filemode (will be None if the object was not reached trough a tree)");
+             "An enums.FileMode constant (or None if the object was not reached through a tree)");
 PyObject *
 Object_filemode__get__(Object *self)
 {
     if (self->entry == NULL)
         Py_RETURN_NONE;
 
-    return PyLong_FromLong(git_tree_entry_filemode(self->entry));
+    return pygit2_enum(FileModeEnum, git_tree_entry_filemode(self->entry));
 }
 
 
@@ -310,9 +290,7 @@ Object_richcompare(PyObject *o1, PyObject *o2, int op)
 }
 
 PyGetSetDef Object_getseters[] = {
-    GETTER(Object, oid),
     GETTER(Object, id),
-    GETTER(Object, hex),
     GETTER(Object, short_id),
     GETTER(Object, type),
     GETTER(Object, type_str),
