@@ -48,6 +48,7 @@ PyObject *FileModeEnum;
 PyObject *FileStatusEnum;
 PyObject *MergeAnalysisEnum;
 PyObject *MergePreferenceEnum;
+PyObject *ReferenceTypeEnum;
 
 extern PyTypeObject RepositoryType;
 extern PyTypeObject OdbType;
@@ -108,6 +109,7 @@ discover_repository(PyObject *self, PyObject *args)
     const char *ceiling_dirs = NULL;
     PyObject *py_repo_path = NULL;
     int err;
+    PyObject *result = NULL;
 
     if (!PyArg_ParseTuple(args, "O&|IO&", PyUnicode_FSConverter, &py_path, &across_fs,
                           PyUnicode_FSConverter, &py_ceiling_dirs))
@@ -121,18 +123,20 @@ discover_repository(PyObject *self, PyObject *args)
     memset(&repo_path, 0, sizeof(git_buf));
     err = git_repository_discover(&repo_path, path, across_fs, ceiling_dirs);
 
-    Py_XDECREF(py_path);
+    if (err == GIT_OK) {
+        py_repo_path = PyUnicode_DecodeFSDefault(repo_path.ptr);
+        git_buf_dispose(&repo_path);
+        result = py_repo_path;
+    } else if (err == GIT_ENOTFOUND) {
+        result = Py_None;
+    } else {
+        result = Error_set_str(err, path);
+    }
+
     Py_XDECREF(py_ceiling_dirs);
+    Py_XDECREF(py_path);
 
-    if (err == GIT_ENOTFOUND)
-        Py_RETURN_NONE;
-    if (err < 0)
-        return Error_set_str(err, path);
-
-    py_repo_path = to_path(repo_path.ptr);
-    git_buf_dispose(&repo_path);
-
-    return py_repo_path;
+    return result;
 };
 
 PyDoc_STRVAR(hashfile__doc__,
@@ -199,6 +203,7 @@ init_file_backend(PyObject *self, PyObject *args)
     unsigned int flags = 0;
     int err = GIT_OK;
     git_repository *repository = NULL;
+    PyObject *result = NULL;
 
     if (!PyArg_ParseTuple(args, "O&|I", PyUnicode_FSConverter, &py_path, &flags))
         return NULL;
@@ -207,25 +212,23 @@ init_file_backend(PyObject *self, PyObject *args)
 
     err = git_repository_open_ext(&repository, path, flags, NULL);
 
-    Py_XDECREF(py_path);
-
-    if (err < 0) {
+    if (err == GIT_OK) {
+        result = PyCapsule_New(repository, "backend", NULL);
+    } else {
+        result = NULL;
         Error_set_str(err, path);
-        goto cleanup;
+
+        if (repository) {
+            git_repository_free(repository);
+        }
+
+        if (err == GIT_ENOTFOUND) {
+            PyErr_Format(GitError, "Repository not found at %s", path);
+        }
     }
 
-    return PyCapsule_New(repository, "backend", NULL);
-
-cleanup:
-    if (repository) {
-        git_repository_free(repository);
-    }
-
-    if (err == GIT_ENOTFOUND) {
-        PyErr_Format(GitError, "Repository not found at %s", path);
-    }
-
-    return NULL;
+    Py_XDECREF(py_path);
+    return result;
 }
 
 
@@ -377,6 +380,7 @@ forget_enums(void)
     Py_CLEAR(FileStatusEnum);
     Py_CLEAR(MergeAnalysisEnum);
     Py_CLEAR(MergePreferenceEnum);
+    Py_CLEAR(ReferenceTypeEnum);
 }
 
 PyDoc_STRVAR(_cache_enums__doc__,
@@ -411,6 +415,7 @@ _cache_enums(PyObject *self, PyObject *args)
     CACHE_PYGIT2_ENUM(FileStatus);
     CACHE_PYGIT2_ENUM(MergeAnalysis);
     CACHE_PYGIT2_ENUM(MergePreference);
+    CACHE_PYGIT2_ENUM(ReferenceType);
 
 #undef CACHE_PYGIT2_ENUM
 
@@ -499,6 +504,8 @@ PyInit__pygit2(void)
     ADD_CONSTANT_INT(m, GIT_OPT_DISABLE_PACK_KEEP_FILE_CHECKS);
     ADD_CONSTANT_INT(m, GIT_OPT_GET_OWNER_VALIDATION);
     ADD_CONSTANT_INT(m, GIT_OPT_SET_OWNER_VALIDATION);
+    ADD_CONSTANT_INT(m, GIT_OPT_GET_MWINDOW_FILE_LIMIT);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_MWINDOW_FILE_LIMIT);
 
     /* Exceptions */
     ADD_EXC(m, GitError, NULL);
@@ -562,12 +569,6 @@ PyInit__pygit2(void)
     ADD_CONSTANT_INT(m, GIT_FILEMODE_BLOB_EXECUTABLE)
     ADD_CONSTANT_INT(m, GIT_FILEMODE_LINK)
     ADD_CONSTANT_INT(m, GIT_FILEMODE_COMMIT)
-    /* Deprecated constants for backward compatibility with old Python code*/
-    ADD_CONSTANT_INT(m, GIT_OBJ_ANY)
-    ADD_CONSTANT_INT(m, GIT_OBJ_COMMIT)
-    ADD_CONSTANT_INT(m, GIT_OBJ_TREE)
-    ADD_CONSTANT_INT(m, GIT_OBJ_BLOB)
-    ADD_CONSTANT_INT(m, GIT_OBJ_TAG)
 
     /*
      * Log
@@ -606,10 +607,6 @@ PyInit__pygit2(void)
     ADD_TYPE(m, Reference)
     ADD_TYPE(m, RefLogEntry)
     ADD_TYPE(m, Note)
-    ADD_CONSTANT_INT(m, GIT_REF_INVALID)
-    ADD_CONSTANT_INT(m, GIT_REF_OID)
-    ADD_CONSTANT_INT(m, GIT_REF_SYMBOLIC)
-    ADD_CONSTANT_INT(m, GIT_REF_LISTALL)
     ADD_CONSTANT_INT(m, GIT_REFERENCES_ALL)
     ADD_CONSTANT_INT(m, GIT_REFERENCES_BRANCHES)
     ADD_CONSTANT_INT(m, GIT_REFERENCES_TAGS)
@@ -622,10 +619,6 @@ PyInit__pygit2(void)
     ADD_CONSTANT_INT(m, GIT_REVSPEC_SINGLE)
     ADD_CONSTANT_INT(m, GIT_REVSPEC_RANGE)
     ADD_CONSTANT_INT(m, GIT_REVSPEC_MERGE_BASE)
-    /* Deprecated enums */
-    ADD_CONSTANT_INT(m, GIT_REVPARSE_SINGLE)
-    ADD_CONSTANT_INT(m, GIT_REVPARSE_RANGE)
-    ADD_CONSTANT_INT(m, GIT_REVPARSE_MERGE_BASE)
 
     /*
      * Worktree
@@ -788,6 +781,7 @@ PyInit__pygit2(void)
     ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_XDG);
     ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_GLOBAL);
     ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_LOCAL);
+    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_WORKTREE);
     ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_APP);
     ADD_CONSTANT_INT(m, GIT_CONFIG_HIGHEST_LEVEL);
 
