@@ -43,6 +43,7 @@ extern PyObject *GitError;
 extern PyTypeObject RefLogEntryType;
 extern PyTypeObject RepositoryType;
 extern PyTypeObject SignatureType;
+extern PyObject *ReferenceTypeEnum;
 
 PyTypeObject ReferenceType;
 
@@ -217,14 +218,15 @@ Reference_rename(Reference *self, PyObject *py_name)
     CHECK_REFERENCE(self);
 
     // Get the C name
-    char *c_name = pgit_encode_fsdefault(py_name);
+    PyObject *tvalue;
+    char *c_name = pgit_borrow_fsdefault(py_name, &tvalue);
     if (c_name == NULL)
         return NULL;
 
     // Rename
     git_reference *new_reference;
     int err = git_reference_rename(&new_reference, self->reference, c_name, 0, NULL);
-    free(c_name);
+    Py_DECREF(tvalue);
     if (err)
         return Error_set(err);
 
@@ -296,7 +298,7 @@ Reference_target__get__(Reference *self)
     if (ret != NULL)
         return ret;
     if (c_name != NULL)
-        return to_path(c_name);
+        return PyUnicode_DecodeFSDefault(c_name);
     return NULL;
 }
 
@@ -365,12 +367,13 @@ Reference_set_target(Reference *self, PyObject *args, PyObject *kwds)
     }
 
     /* Case 2: Symbolic */
-    char *c_name = pgit_encode_fsdefault(py_target);
+    PyObject *tvalue;
+    char *c_name = pgit_borrow_fsdefault(py_target, &tvalue);
     if (c_name == NULL)
         return NULL;
 
     err = git_reference_symbolic_set_target(&new_ref, self->reference, c_name, message);
-    free(c_name);
+    Py_DECREF(tvalue);
     if (err < 0)
         goto error;
 
@@ -391,7 +394,7 @@ PyObject *
 Reference_name__get__(Reference *self)
 {
     CHECK_REFERENCE(self);
-    return to_path(git_reference_name(self->reference));
+    return PyUnicode_DecodeFSDefault(git_reference_name(self->reference));
 }
 
 PyDoc_STRVAR(Reference_raw_name__doc__, "The full name of the reference (Bytes).");
@@ -410,7 +413,7 @@ PyObject *
 Reference_shorthand__get__(Reference *self)
 {
     CHECK_REFERENCE(self);
-    return to_path(git_reference_shorthand(self->reference));
+    return PyUnicode_DecodeFSDefault(git_reference_shorthand(self->reference));
 }
 
 PyDoc_STRVAR(Reference_raw_shorthand__doc__,
@@ -424,7 +427,7 @@ Reference_raw_shorthand__get__(Reference *self)
 }
 
 PyDoc_STRVAR(Reference_type__doc__,
-    "Type, either GIT_REF_OID or GIT_REF_SYMBOLIC.");
+    "An enums.ReferenceType constant (either OID or SYMBOLIC).");
 
 PyObject *
 Reference_type__get__(Reference *self)
@@ -433,7 +436,8 @@ Reference_type__get__(Reference *self)
 
     CHECK_REFERENCE(self);
     c_type = git_reference_type(self->reference);
-    return PyLong_FromLong(c_type);
+
+    return pygit2_enum(ReferenceTypeEnum, c_type);
 }
 
 
