@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2013 The pygit2 contributors
+ * Copyright 2010-2024 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -33,42 +33,44 @@
 #include <git2.h>
 #include "types.h"
 
+typedef enum {
+	GIT_REFERENCES_ALL = 0,
+	GIT_REFERENCES_BRANCHES = 1,
+	GIT_REFERENCES_TAGS = 2,
+} git_reference_iterator_return_t;
+
+PyObject *wrap_repository(git_repository *c_repo);
+
 int  Repository_init(Repository *self, PyObject *args, PyObject *kwds);
 int  Repository_traverse(Repository *self, visitproc visit, void *arg);
 int  Repository_clear(Repository *self);
-int  Repository_contains(Repository *self, PyObject *value);
 
-git_odb_object*
-Repository_read_raw(git_repository *repo, const git_oid *oid, size_t len);
-
-PyObject* Repository_head(Repository *self);
-PyObject* Repository_getitem(Repository *self, PyObject *value);
-PyObject* Repository_read(Repository *self, PyObject *py_hex);
-PyObject* Repository_write(Repository *self, PyObject *args);
-PyObject* Repository_get_index(Repository *self, void *closure);
-PyObject* Repository_get_path(Repository *self, void *closure);
-PyObject* Repository_get_workdir(Repository *self, void *closure);
-PyObject* Repository_get_config(Repository *self, void *closure);
 PyObject* Repository_walk(Repository *self, PyObject *args);
 PyObject* Repository_create_blob(Repository *self, PyObject *args);
-PyObject* Repository_create_blob_fromfile(Repository *self, PyObject *args);
+PyObject* Repository_create_blob_fromdisk(Repository *self, PyObject *args);
 PyObject* Repository_create_commit(Repository *self, PyObject *args);
+PyObject* Repository_create_commit_string(Repository *self, PyObject *args);
+PyObject* Repository_create_commit_with_signature(Repository *self, PyObject *args);
 PyObject* Repository_create_tag(Repository *self, PyObject *args);
 PyObject* Repository_create_branch(Repository *self, PyObject *args);
-PyObject* Repository_listall_references(Repository *self, PyObject *args);
+PyObject* Repository_references_iterator_init(Repository *self, PyObject *args);
+PyObject* Repository_references_iterator_next(Repository *self, PyObject *args);
 PyObject* Repository_listall_branches(Repository *self, PyObject *args);
 PyObject* Repository_lookup_reference(Repository *self, PyObject *py_name);
+PyObject* Repository_add_worktree(Repository *self, PyObject *args);
+PyObject* Repository_lookup_worktree(Repository *self, PyObject *py_name);
+PyObject* Repository_list_worktrees(Repository *self, PyObject *args);
 
-PyObject*
-Repository_create_reference(Repository *self, PyObject *args, PyObject* kw);
+PyObject* Repository_create_reference_direct(Repository *self, PyObject *args, PyObject* kw);
+PyObject* Repository_create_reference_symbolic(Repository *self, PyObject *args, PyObject* kw);
 
-PyObject* Repository_packall_references(Repository *self,  PyObject *args);
-PyObject* Repository_status(Repository *self);
+PyObject* Repository_compress_references(Repository *self);
+PyObject* Repository_status(Repository *self, PyObject *args, PyObject *kw);
 PyObject* Repository_status_file(Repository *self, PyObject *value);
 PyObject* Repository_TreeBuilder(Repository *self, PyObject *args);
 
-PyObject* Repository_blame(Repository *self, PyObject *args, PyObject *kwds);
-
-PyObject* Repository_merge(Repository *self, PyObject *py_oid);
+PyObject* Repository_cherrypick(Repository *self, PyObject *py_oid);
+PyObject* Repository_apply(Repository *self, PyObject *py_diff, PyObject *kwds);
+PyObject* Repository_merge_analysis(Repository *self, PyObject *args);
 
 #endif
