@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2013 The pygit2 contributors
+ * Copyright 2010-2024 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -45,13 +45,15 @@ TreeBuilder_dealloc(TreeBuilder *self)
 
 
 PyDoc_STRVAR(TreeBuilder_insert__doc__,
-    "insert(name, oid, attr)\n"
+    "insert(name: str, oid: Oid, attr: FileMode)\n"
     "\n"
     "Insert or replace an entry in the treebuilder.\n"
     "\n"
-    "attr available values are GIT_FILEMODE_BLOB,\n"
-    "     GIT_FILEMODE_BLOB_EXECUTABLE, GIT_FILEMODE_TREE,\n"
-    "     GIT_FILEMODE_LINK and GIT_FILEMODE_COMMIT.");
+    "Parameters:\n"
+    "\n"
+    "attr\n"
+    "    Available values are FileMode.BLOB, FileMode.BLOB_EXECUTABLE,\n"
+    "    FileMode.TREE, FileMode.LINK and FileMode.COMMIT.\n");
 
 PyObject *
 TreeBuilder_insert(TreeBuilder *self, PyObject *args)
@@ -88,7 +90,7 @@ TreeBuilder_write(TreeBuilder *self)
     int err;
     git_oid oid;
 
-    err = git_treebuilder_write(&oid, self->repo->repo, self->bld);
+    err = git_treebuilder_write(&oid, self->bld);
     if (err < 0)
         return Error_set(err);
 
@@ -97,51 +99,49 @@ TreeBuilder_write(TreeBuilder *self)
 
 
 PyDoc_STRVAR(TreeBuilder_get__doc__,
-    "get(name) -> TreeEntry\n"
+    "get(name: str) -> Object\n"
     "\n"
-    "Return the TreeEntry for the given name, or None if there is not.");
+    "Return the Object for the given name, or None if there is not.");
 
 PyObject *
 TreeBuilder_get(TreeBuilder *self, PyObject *py_filename)
 {
-    char *filename;
-    const git_tree_entry *entry;
-
-    filename = py_path_to_c_str(py_filename);
+    PyObject *tvalue;
+    char *filename = pgit_borrow_fsdefault(py_filename, &tvalue);
     if (filename == NULL)
         return NULL;
 
-    entry = git_treebuilder_get(self->bld, filename);
-    free(filename);
-    if (entry == NULL)
+    const git_tree_entry *entry_src = git_treebuilder_get(self->bld, filename);
+    Py_DECREF(tvalue);
+    if (entry_src == NULL)
         Py_RETURN_NONE;
 
-    entry = git_tree_entry_dup(entry);
-    if (entry == NULL) {
+    git_tree_entry *entry;
+    if (git_tree_entry_dup(&entry, entry_src) < 0) {
         PyErr_SetNone(PyExc_MemoryError);
         return NULL;
     }
-    return (PyObject*)wrap_tree_entry(entry);
+
+    return treeentry_to_object(entry, self->repo);
 }
 
 
 PyDoc_STRVAR(TreeBuilder_remove__doc__,
-    "remove(name)\n"
+    "remove(name: str)\n"
     "\n"
     "Remove an entry from the builder.");
 
 PyObject *
 TreeBuilder_remove(TreeBuilder *self, PyObject *py_filename)
 {
-    char *filename = py_path_to_c_str(py_filename);
-    int err = 0;
-
+    PyObject *tvalue;
+    char *filename = pgit_borrow_fsdefault(py_filename, &tvalue);
     if (filename == NULL)
         return NULL;
 
-    err = git_treebuilder_remove(self->bld, filename);
-    free(filename);
-    if (err < 0)
+    int err = git_treebuilder_remove(self->bld, filename);
+    Py_DECREF(tvalue);
+    if (err)
         return Error_set(err);
 
     Py_RETURN_NONE;
